import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

/*
 * Created on Feb 11, 2004
 */

/**
 * @author Kent
 */
public class gui extends JFrame{
	// array of objects that represent the button-label combinations
	private data[] items; 
	private int itemcount;  // how many buttons are there
	private int maxcount;
	private JPanel upper; // holds the buttons
	private JPanel lower; // holds the winner button
	private JButton winner;
	
	public gui(int n) 
	{
		items = new data[n];
		itemcount = 0;
		maxcount = n;	
		upper = new JPanel();
		upper.setLayout(new FlowLayout());
		lower = new JPanel();
		lower.setLayout(new FlowLayout());
		
		winner = new JButton("Winner");
		winner.addActionListener( new winnerlistener());
		lower.add(winner);
		
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(upper,BorderLayout.NORTH);
		getContentPane().add(lower,BorderLayout.SOUTH);
		
		setSize(400,400);      // set frame size
		setVisible(true);   // make sure we can see it
	} // gui constructor
	
	public void additem(String n) throws TooManyException{
		if(itemcount == maxcount) throw new TooManyException(maxcount);
		items[itemcount] = new data(n);
		upper.add(items[itemcount].getpanel());
		itemcount++;
		show();
	}  // additem

	/*
	 * For each button, there is a data object.
	 * This creates the panel that holds the button and  its
	 * label. It also contains the number of times the button
	 * was pressed. The listener for this object is an inner class
	 * on this inner class.
	 */
	private class data {
		private int count;   // how many times was this button pressed
		private String name;  // button name
		private JLabel showcount;  // displays the count or winner
		private JPanel panel;   // holds the button and label
		private JButton button;
		
		// the parameter is the name for the button
		public data(String n)
		{
			count = 0;
			name = new String(n);
			panel = new JPanel();
			button = new JButton(name);
			showcount = new JLabel("0"); // starts at 0
			
			// using a BorderLayout gets us the button above the
			// label. Could have used a BoxLayout as well
			panel.setLayout(new BorderLayout());
			panel.add(button,BorderLayout.NORTH);
			panel.add(showcount,BorderLayout.SOUTH);
			button.addActionListener( new buttonlistener());
		} // data constructor
		
		public JPanel getpanel() { return panel;}
		public int getcount() { return count;}
		//
		// update the label to show this as the winner.
		// This could have been done using a get and setLabel() method
		// here and the other code in the winner listener
		public void winner() {
			String label = new String(showcount.getText());
			showcount.setText( "Winner! " + label);
		} // winner
		
		// handles the events on this button
		// Could have made this an inner class on the gui class
		// and used only one object for all the buttons.
		// But I wanted to show nested inner classes
		// and it made the logic in the listener a little easier
		// since this listener would only be called by this button.
		private class buttonlistener implements ActionListener {
			public void actionPerformed(ActionEvent event)
		    {
		    	Object s = event.getSource();
		    	if(s == button){
		    		count++;
		    		showcount.setText(Integer.toString(count));
				// force the re-draw or we don't see the change
		    		panel.repaint(); 
		    	}
		    } // actionPerformed
		} // buttonlistener
	} // data class
	
	// this handles presses of the winner button
	private class winnerlistener implements ActionListener {
		public void actionPerformed(ActionEvent event)
		{
			Object s = event.getSource();
			// search the array of data objects, checking
			// for the largest count
			if(s == winner){
				int max = 0, location=0;
				for(int i = 0; i< itemcount; i++) {
					if (items[i].getcount() > max) {
						max = items[i].getcount();
						location = i;
					}	
				} // for
				// when the for loop is done, location holds
				// the array index of the data object with the
				// largest count. Its winner() method will
				// update the label
				items[location].winner();
				System.out.println("the winner is " + location);
			}
		} // actionPerformed
	} // winnerlistener
} // gui class
